/*
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowledgement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgement may appear in the software itself,
 *    if and wherever such third-party acknowledgements normally appear.
 *
 * 4. The names "The Jakarta Project", "Commons", "Jexl" and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package org.apache.commons.jexl.context;

import org.apache.commons.jexl.JexlContext;
import org.apache.commons.jexl.CheckResults;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.HashSet;

/**
 *  Implementation of JexlContext based on a HashMap. This also implements
 *  CheckResults. It is not thread-safe since it stores information about
 *  what variables were requested by the {@link #get(Object)} method. For
 *  this reason it should always be cloned from the real JexlContext. The
 *  {@link #CheckingContext(JexlContext) constructor} will do this for you.
 *
 *  @author <a href="mailto:bill@logicalcobwebs.co.uk">Bill Horsman</a>
 *  @version $Id: $
 */
public class CheckingContext extends HashMap implements JexlContext, CheckResults
{

    private Set knownVariables = new HashSet();
    private Set unknownVariables = new HashSet();
    private Set usedVariables = new HashSet();
    private Object answer;

    /**
     * Constructor that clones another JexlContext. It is a shallow clone and
     * just copies over all the objects from the map.
     * @param originalContext the context we are cloning
     */
    public CheckingContext(JexlContext originalContext)
    {
        this.putAll(originalContext.getVars());
    }

    /**
     * As well as getting the value (as normal) it also keeps track of what was asked
     * for and whether it was found within this context
     * @param key the variable name
     * @return the variable value
     */
    public Object get(Object key)
    {
        if (containsKey(key)) {
            knownVariables.add(key);
        }
        else {
            unknownVariables.add(key);
        }
        usedVariables.add(key);
        return super.get(key);
    }

    /**
     * @see CheckResults#getKnownVariables()
     */
    public String[] getKnownVariables()
    {
        return (String[]) knownVariables.toArray(new String[knownVariables.size()]);
    }

    /**
     * @see CheckResults#isKnownVariables()
     */
    public boolean isKnownVariables()
    {
        return (knownVariables.size() > 0);
    }

    /**
     * @see CheckResults#getUnknownVariables()
     */
    public String[] getUnknownVariables()
    {
        return (String[]) unknownVariables.toArray(new String[unknownVariables.size()]);
    }

    /**
     * @see CheckResults#isUnknownVariables()
     */
    public boolean isUnknownVariables()
    {
        return (unknownVariables.size() > 0);
    }

    /**
     * @see CheckResults#getUsedVariables()
     */
    public String[] getUsedVariables()
    {
        return (String[]) usedVariables.toArray(new String[usedVariables.size()]);
    }

    /**
     * @see CheckResults#isUsedVariables()
     */
    public boolean isUsedVariables()
    {
        return (usedVariables.size() > 0);
    }

    /**
     * Resets the unknown, known and used variable list.
     */
    public void reset()
    {
        unknownVariables.clear();
        knownVariables.clear();
        usedVariables.clear();
    }

    /**
     * Overrides clear by calling {@link #reset()} as well.
     */
    public void clear()
    {
        reset();
        super.clear();
    }

    /**
     * @see JexlContext#setVars(Map)
     */
    public void setVars(Map vars)
    {
        this.clear();
        this.putAll(vars);
    }

    /**
     * @see JexlContext#getVars()
     */
    public Map getVars()
    {
        return this;
    }

    /**
     * @see CheckResults#getAnswer()
     */
    public Object getAnswer()
    {
        return answer;
    }

    /**
     * @see CheckResults#getAnswer()
     */
    public void setAnswer(Object answer)
    {
        this.answer = answer;
    }
}
